# pylint: skip-file

"""
Wraps the srt file.
"""

# designed to be run in an isolated environment.

import shutil
import sys
import tempfile
from argparse import ArgumentParser, Namespace
from pathlib import Path

from srtranslator import SrtFile  # type: ignore


# srtranslator==0.2.6
def srt_wrap(srt_file: Path) -> str:
    """Wrap lines in a srt file."""
    with tempfile.TemporaryDirectory() as tempdir:
        out_file = Path(tempdir) / "out.srt"
        shutil.copy(srt_file, out_file)
        srt = SrtFile(str(srt_file))
        srt.wrap_lines()
        srt.save(str(out_file))
        string = out_file.read_text(encoding="utf-8")
        return string


def create_args() -> Namespace:
    """Create args."""
    parser = ArgumentParser(description="Wrap lines in a srt file.")
    parser.add_argument("input_srt", help="The srt file to wrap.")
    args = parser.parse_args()
    return args


def main() -> int:
    """Main entry point for the command line tool."""
    args = create_args()
    out = srt_wrap(args.input_srt)
    sys.stdout.write(out)
    return 0


if __name__ == "__main__":
    sys.exit(main())



"""
Tests transcribe_anything
"""

# pylint: disable=bad-option-value,useless-option-value,no-self-use,protected-access,R0801
# flake8: noqa E501

import json
import os
import sys
import tempfile
import unittest

IS_MACOS = sys.platform == "darwin"

HERE = os.path.abspath(os.path.dirname(__file__))


class BadVideoTitleTester(unittest.TestCase):
    """Tester for transcribe anything."""

    @unittest.skipIf(IS_MACOS, "Skipping test on MacOS")
    def test_local_file(self) -> None:
        """Check that the command works on a local file."""
        prevdir = os.getcwd()
        with tempfile.TemporaryDirectory() as tmpdir:
            try:
                print(f"Using temporary directory {tmpdir}")
                os.chdir(tmpdir)
                os.system("transcribe_anything --query-gpu-json-path info.json")
                self.assertTrue(os.path.exists("info.json"))
                # parsing test
                with open("info.json", encoding="utf-8", mode="rt") as fd:
                    info = json.load(fd)
                self.assertTrue(info is not None)
            finally:
                os.chdir(prevdir)


if __name__ == "__main__":
    unittest.main()



"""
Installs whisper in an isolated environment.
"""

import os
from pathlib import Path

from isolated_environment import IsolatedEnvironment  # type: ignore

TENSOR_VERSION = "2.1.2"
CUDA_VERSION = "cu121"
TENSOR_CUDA_VERSION = f"{TENSOR_VERSION}+{CUDA_VERSION}"
EXTRA_INDEX_URL = f"https://download.pytorch.org/whl/{CUDA_VERSION}"

HERE = Path(os.path.abspath(os.path.dirname(__file__)))


def unit_test() -> None:
    """Unit test."""
    from tempfile import TemporaryDirectory  # pylint: disable=import-outside-toplevel

    with TemporaryDirectory() as tmpdir:
        print(f"Using temporary directory {tmpdir}")
        iso_env = IsolatedEnvironment(HERE / "transcribe_anything_env")
        iso_env.install_environment()
        iso_env.pip_install(
            package=f"torch=={TENSOR_VERSION}",
            build_options=EXTRA_INDEX_URL,
            full_isolation=False,
        )
        iso_env.pip_install(
            package="openai-whisper", build_options=None, full_isolation=False
        )
        iso_env.run(["whisper", "--help"])


if __name__ == "__main__":
    unit_test()



"""
Tests transcribe_anything
"""

# pylint: disable=bad-option-value,useless-option-value,no-self-use,protected-access,R0801
# flake8: noqa E501

import unittest

from transcribe_anything.insanley_fast_whisper_reqs import get_environment


class InsanelFastWhisperTesterDeps(unittest.TestCase):
    """Tester for transcribe anything."""

    def test(self) -> None:
        """Check that the command works on a local file."""
        get_environment()


if __name__ == "__main__":
    unittest.main()



# pylint: skip-file
# flake8: noqa

"""
Runs whisper api.
"""

import json  # type: ignore
import subprocess
import sys
import tempfile
import time
import traceback
import warnings
import wave
from pathlib import Path
from typing import Any, Optional

import webvtt  # type: ignore

from transcribe_anything.cuda_available import CudaInfo
from transcribe_anything.generate_speaker_json import generate_speaker_json
from transcribe_anything.insanley_fast_whisper_reqs import get_environment

HERE = Path(__file__).parent
CUDA_INFO: Optional[CudaInfo] = None


def get_cuda_info() -> CudaInfo:
    """Get the computing device."""
    global CUDA_INFO  # pylint: disable=global-statement
    if CUDA_INFO is None:
        env = get_environment()
        py_file = HERE / "cuda_available.py"
        # tempfile
        with tempfile.TemporaryDirectory() as dir_name:
            temp = Path(dir_name) / "stdout.txt"
            abs_name = temp.absolute()
            cp: subprocess.CompletedProcess = subprocess.run(
                ["python", py_file, "-o", abs_name],
                shell=False,
                check=False,
                env=env,
                universal_newlines=True,
                text=True,
            )
            # stdout = cp.stdout
            stdout = temp.read_text(encoding="utf-8")
            try:
                CUDA_INFO = CudaInfo.from_json_str(stdout)
            except json.JSONDecodeError as exc:
                raise ValueError(f"Failed to decode json: {exc}") from exc
            assert (
                CUDA_INFO is not None
            ), f"Expected CUDA_INFO to be set, but the stdout was {stdout}"
    return CUDA_INFO


def get_device_id() -> str:
    """Get the device id."""
    # on mac, we just return "mps"
    if sys.platform == "darwin":
        return "mps"
    cuda_info = get_cuda_info()
    if not cuda_info.cuda_available:
        raise ValueError("CUDA is not available.")
    device_id = cuda_info.cuda_devices[0].device_id
    return f"{device_id}"


def get_batch_size() -> int | None:
    """Returns the batch size."""
    if sys.platform == "darwin":
        return 4
    return None


def convert_time_to_srt_format(timestamp: float) -> str:
    """Converts timestamp in seconds to SRT time format (hours:minutes:seconds,milliseconds)."""
    hours, remainder = divmod(timestamp, 3600)
    minutes, seconds = divmod(remainder, 60)
    milliseconds = int((seconds % 1) * 1000)
    seconds = int(seconds)
    return f"{int(hours):02}:{int(minutes):02}:{seconds:02},{milliseconds:03}"


def convert_to_webvtt(srt_file: Path, out_webvtt_file: Path) -> None:
    """Convert to webvtt format."""
    STYLE_ELEMENT = """STYLE
    ::::cue {
    line: 80%;
    }
    """
    assert srt_file.suffix == ".srt"
    assert out_webvtt_file.suffix == ".vtt"
    webvtt.from_srt(str(srt_file)).save(str(out_webvtt_file))
    content = out_webvtt_file.read_text(encoding="utf-8")
    content = content.replace("WEBVTT\n\n", f"WEBVTT\n\n{STYLE_ELEMENT}")
    out_webvtt_file.write_text(content, encoding="utf-8")


def convert_json_to_srt(json_data: dict[str, Any], duration: float) -> str:
    """Converts JSON data from speech-to-text tool to SRT format."""
    srt_content = ""
    num_chunks = len(json_data["chunks"])
    for index, chunk in enumerate(json_data["chunks"], start=1):
        # start_time, end_time = chunk["timestamp"]
        time_pair = chunk["timestamp"]
        start_time = time_pair[0]
        end_time = time_pair[1]
        try:
            if start_time is None and end_time is None:
                print(
                    f"Skipping chunk {index} because both start and end time are None."
                )
                stack_trace = traceback.format_stack()
                print("Stack trace: ", stack_trace)
                continue
            if end_time is None:
                # assert index == num_chunks
                if index != num_chunks:
                    print(
                        f"Setting end time to duration because it's None for chunk {index}."
                    )
                end_time = duration  # Sometimes happens at the end
            try:
                start_time_str = convert_time_to_srt_format(start_time)
            except Exception as exc:
                print(f"Failed to convert start time {start_time} to srt format: {exc}")
                stack_trace = traceback.format_stack()
                print("Stack trace: ", stack_trace)
                continue
            try:
                end_time_str = convert_time_to_srt_format(end_time)
            except Exception as exc:
                print(f"Failed to convert end time {end_time} to srt format: {exc}")
                stack_trace = traceback.format_stack()
                print("Stack trace: ", stack_trace)
                continue
        except Exception as exc:
            print(f"Failed to convert times for chunk {index}: {exc}")
            stack_trace = traceback.format_stack()
            print("Stack trace: ", stack_trace)
            continue
        try:
            text = str(chunk["text"]).strip()
            srt_content += f"{index}\n{start_time_str} --> {end_time_str}\n{text}\n\n"
        except Exception as exc:
            print(f"Failed to add chunk {index} to srt content: {exc}")
            stack_trace = traceback.format_stack()
            print("Stack trace: ", stack_trace)
            continue
    return srt_content


def convert_json_to_text(json_data: dict[str, Any]) -> str:
    """Converts JSON data from speech-to-text tool to text."""
    return json_data["text"]


def get_wave_duration(wave_file: Path) -> float:
    """Returns the duration of a wave file."""
    with wave.open(str(wave_file), "rb") as wav:
        frames = wav.getnframes()
        rate = wav.getframerate()
        duration = frames / float(rate)
        return duration


def trim_text_chunks(json_data: dict[str, Any]) -> None:
    """'text' chunks seem to have an extra space at the beginning, remove it."""

    # visit all the nodes in the json data, when we see one that has a 'text' key,
    # then apply a trim.
    def visit(node: dict[str, Any]) -> None:
        if isinstance(node, dict):
            if "text" in node:
                node["text"] = node["text"].strip()
            for key, value in node.items():
                visit(value)
        elif isinstance(node, list):
            for item in node:
                visit(item)

    visit(json_data)


def run_insanely_fast_whisper(
    input_wav: Path,
    model: str,
    output_dir: Path,
    task: str,
    language: str,
    hugging_face_token: str | None = None,
    other_args: list[str] | None = None,
) -> None:
    """Runs insanely fast whisper."""
    env = get_environment()
    device_id = get_device_id()
    cmd_list = []
    output_dir.mkdir(parents=True, exist_ok=True)
    outfile = output_dir / "out.json"
    if "/" not in model:
        # Assume it's not a namespace model, so add the namespace.
        model = f"openai/whisper-{model}"
    wave_duration = get_wave_duration(input_wav)
    if sys.platform == "win32":
        # Set the text mode to UTF-8 on Windows.
        cmd_list.extend(["chcp", "65001", "&&"])
    cmd_list += [
        "insanely-fast-whisper",
        "--file-name",
        str(input_wav),
        "--device-id",
        f"{device_id}",
    ]
    if model:
        cmd_list += [
            "--model-name",
            model,
        ]
    cmd_list += [
        "--task",
        task,
        "--transcript-path",
        str(outfile),
    ]
    if hugging_face_token:
        cmd_list += ["--hf_token", hugging_face_token]
    if language:
        cmd_list += ["--language", language]
    batch_size = get_batch_size()
    if batch_size is not None:
        cmd_list += ["--batch-size", f"{batch_size}"]
    if other_args:
        cmd_list.extend(other_args)
    # Remove the empty strings.
    cmd_list = [x.strip() for x in cmd_list if x.strip()]
    cmd = " ".join(cmd_list)
    sys.stderr.write(f"Running:\n  {cmd}\n")
    proc = subprocess.Popen(  # pylint: disable=consider-using-with
        cmd,
        shell=True,
        universal_newlines=True,
        encoding="utf-8",
        env=env,
    )
    while True:
        rtn = proc.poll()
        if rtn is None:
            time.sleep(0.25)
            continue
        if rtn != 0:
            msg = f"Failed to execute {cmd}\n "
            raise OSError(msg)
        break
    assert outfile.exists(), f"Expected {outfile} to exist."
    json_text = outfile.read_text(encoding="utf-8")
    json_data = json.loads(json_text)
    trim_text_chunks(json_data)
    json_data_str = json.dumps(json_data, indent=2)

    if hugging_face_token:
        # Speaker diarization is active so generate the file
        try:
            speaker_json = generate_speaker_json(json_data)
            speaker_json_str = json.dumps(speaker_json, indent=2)
            speaker_json_file = output_dir / "speaker.json"
            speaker_json_file.write_text(speaker_json_str, encoding="utf-8")
        except Exception as exc:
            warnings.warn(f"Failed to generate speaker json beause of exception: {exc}")

    # now write the pretty formatted json data back to the text file.
    outfile.write_text(json_data_str, encoding="utf-8")
    try:
        srt_content = convert_json_to_srt(json_data, wave_duration)
        srt_file = output_dir / "out.srt"
    except Exception as exc:
        print(f"Failed to convert to srt: {exc}")
        print("Json data: ", json_data_str)
        error_file = Path("transcribe-anything-error.json")
        error_file.write_text(json_text, encoding="utf-8")
        raise
    try:
        txt_content = convert_json_to_text(json_data)
    except Exception as exc:
        error_file = Path("transcribe-anything-error.json")
        error_file.write_text(json_text, encoding="utf-8")
        raise
    # Disable srt_wrapping because it breaks conversion to webvtt.
    srt_file.write_text(srt_content, encoding="utf-8")
    # srt_wrap(srt_file)
    # srt_content = srt_file.read_text(encoding="utf-8")
    txt_file = output_dir / "out.txt"
    txt_file.write_text(txt_content, encoding="utf-8")
    convert_to_webvtt(srt_file, output_dir / "out.vtt")
    # print srt file
    print(srt_content)



"""
Utilities for srt translation, including srt wrap.
"""

import subprocess
import warnings
from pathlib import Path
from typing import Any

from isolated_environment import isolated_environment  # type: ignore

HERE = Path(__file__).parent
WRAP_SRT_PY = HERE / "srt_wrap.py"


def get_environment() -> dict[str, Any]:
    """Returns the environment."""
    venv_dir = HERE / "venv" / "srttranslator"
    env = isolated_environment(
        venv_dir, ["srtranslator==0.2.6", "requests==2.28.1", "urllib3==1.26.13"]
    )
    return env


def srt_wrap_to_string(srt_file: Path) -> str:
    """Wrap lines in a srt file."""
    env = get_environment()
    process = subprocess.run(
        ["python", str(WRAP_SRT_PY), str(srt_file)],
        env=env,
        capture_output=True,
        text=True,
        shell=False,
        check=True,
    )
    out = process.stdout
    return out


def srt_wrap(srt_file: Path) -> None:
    """Wrap lines in a srt file."""
    try:
        assert WRAP_SRT_PY.exists()
        out = srt_wrap_to_string(srt_file)
        srt_file.write_text(out, encoding="utf-8")
    except subprocess.CalledProcessError as exc:
        warnings.warn(f"Failed to run srt_wrap: {exc}")
        return



"""
Tests transcribe_anything
"""

# pylint: disable=bad-option-value,useless-option-value,no-self-use,protected-access,R0801
# flake8: noqa E501

import os
import shutil
import unittest

from transcribe_anything.api import transcribe

HERE = os.path.abspath(os.path.dirname(__file__))
LOCALFILE_DIR = os.path.join(HERE, "localfile")
TESTS_DATA_DIR = os.path.join(LOCALFILE_DIR, "text_video", "en")


class TranscribeAnythingApiEmbedTester(unittest.TestCase):
    """Tester for transcribe anything."""

    def test_local_file(self) -> None:
        """Check that the command works on a local file."""
        shutil.rmtree(TESTS_DATA_DIR, ignore_errors=True)
        vidfile = os.path.join(LOCALFILE_DIR, "video.mp4")
        prev_dir = os.getcwd()
        os.chdir(LOCALFILE_DIR)
        transcribe(url_or_file=vidfile, language="en", model="tiny", embed=True)
        os.chdir(prev_dir)
        expected_paths = [
            TESTS_DATA_DIR,
            os.path.join(TESTS_DATA_DIR, "out.txt"),
            os.path.join(TESTS_DATA_DIR, "out.srt"),
            os.path.join(TESTS_DATA_DIR, "out.vtt"),
        ]
        for expected_path in expected_paths:
            self.assertTrue(
                os.path.exists(expected_path),
                f"expected path {expected_path} not found",
            )


if __name__ == "__main__":
    unittest.main()



"""
Requirements for the insanely fast whisper.
"""

import sys
from pathlib import Path
from typing import Any

from isolated_environment import isolated_environment  # type: ignore

from transcribe_anything.util import has_nvidia_smi

HERE = Path(__file__).parent

# Set the versions
TENSOR_VERSION = "2.1.2"
CUDA_VERSION = "cu121"
TENSOR_CUDA_VERSION = f"{TENSOR_VERSION}+{CUDA_VERSION}"
EXTRA_INDEX_URL = f"https://download.pytorch.org/whl/{CUDA_VERSION}"


def get_current_python_version() -> str:
    """Returns the current python version."""
    return f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"


# for insanely fast whisper, use:
#   pipx install insanely-fast-whisper --python python3.11


def get_environment() -> dict[str, Any]:
    """Returns the environment."""
    venv_dir = HERE / "venv" / "insanely_fast_whisper"
    deps = [
        "openai-whisper",
        "insanely-fast-whisper==0.0.13 --ignore-requires-python",
        "torchaudio==2.1.2",
        "pytorch-lightning==2.1.4",
        "torchmetrics~=1.3.0",
        "srtranslator==0.2.6",
    ]
    if has_nvidia_smi():
        deps.append(f"torch=={TENSOR_CUDA_VERSION} --extra-index-url {EXTRA_INDEX_URL}")
    else:
        deps.append(f"torch=={TENSOR_VERSION}")
    if sys.platform != "darwin":
        # Add the windows specific dependencies.
        deps.append("intel-openmp==2024.0.2")
    env = isolated_environment(venv_dir, deps)
    return env



"""
Tests transcribe_anything
"""

# pylint: disable=bad-option-value,useless-option-value,no-self-use,protected-access,R0801
# flake8: noqa E501

import json
import unittest
from pathlib import Path

from transcribe_anything.insanely_fast_whisper import (
    convert_json_to_srt,
    convert_json_to_text,
)

HERE = Path(__file__).parent
LOCALFILE_DIR = HERE / "localfile"
PROBLEM_JSON = LOCALFILE_DIR / "problem.json"

EXAMPLE_JSON = """
{
  "speakers": [],
  "chunks": [
    {
      "timestamp": [
        0,
        3.24
      ],
      "text": " Oh wow, I'm so nervous."
    },
    {
      "timestamp": [
        3.24,
        6.56
      ],
      "text": " Gosh, these lights are so bright."
    },
    {
      "timestamp": [
        6.56,
        8.52
      ],
      "text": " Is this mic on?"
    },
    {
      "timestamp": [
        8.52,
        9.52
      ],
      "text": " Is there even a mic?"
    }
  ],
  "text": " Oh wow, I'm so nervous. Gosh, these lights are so bright. Is this mic on? Is there even a mic?"
}
"""

EXPECTED_SRT_FILE = """
1
00:00:00,000 --> 00:00:03,240
Oh wow, I'm so nervous.

2
00:00:03,240 --> 00:00:06,559
Gosh, these lights are so bright.

3
00:00:06,559 --> 00:00:08,519
Is this mic on?

4
00:00:08,519 --> 00:00:09,519
Is there even a mic?
""".strip()

EXPECTED_TXT_FILE = """
Oh wow, I'm so nervous. Gosh, these lights are so bright. Is this mic on? Is there even a mic?
""".strip()


class JsonToSrtTester(unittest.TestCase):
    """Tester for transcribe anything."""

    def test_json_to_srt(self) -> None:
        """Check that the command works on a local file."""
        data = json.loads(EXAMPLE_JSON)
        out = convert_json_to_srt(data, 9.0)
        self.assertIn(EXPECTED_SRT_FILE, out)

    def test_json_to_txt(self) -> None:
        """Check that the command works on a local file."""
        data = json.loads(EXAMPLE_JSON)
        out = convert_json_to_text(data)
        self.assertIn(EXPECTED_TXT_FILE, out)


if __name__ == "__main__":
    unittest.main()



"""
    Entry point for running the transcribe-anything prgram.
"""

# flake8: noqa E501
# pylint: disable=too-many-branches,import-outside-toplevel

import argparse
import json
import os
import sys
import traceback
from pathlib import Path

# appdirs is used to get the cache directory
from appdirs import user_cache_dir  # type: ignore

from transcribe_anything.parse_whisper_options import parse_whisper_options
from transcribe_anything.whisper import get_computing_device

HERE = Path(os.path.abspath(os.path.dirname(__file__)))
WHISPER_OPTIONS = HERE / "WHISPER_OPTIONS.json"

os.environ["PYTHONIOENCODING"] = "utf-8"

WHISPER_MODEL_OPTIONS = [
    "tiny",
    "tiny.en",
    "base",
    "base.en",
    "small",
    "small.en",
    "medium",
    "medium.en",
    "large-legacy",
    "large",
    "large-v2",
    "large-v3",
    "distil-whisper/distil-large-v2",
]


def get_whisper_options() -> dict:
    """Get whisper options.""" ""
    if not WHISPER_OPTIONS.exists():
        whisper_options = parse_whisper_options()
        string = json.dumps(whisper_options, indent=4)
        WHISPER_OPTIONS.write_text(string)
        return whisper_options
    file_age = os.path.getmtime(WHISPER_OPTIONS)
    if file_age > 60 * 60 * 24 * 7:  # 1 week
        whisper_options = parse_whisper_options()
        string = json.dumps(whisper_options, indent=4)
        WHISPER_OPTIONS.write_text(string)
    return whisper_options


def parse_arguments() -> argparse.Namespace:
    """Parse arguments."""
    whisper_options = get_whisper_options()
    device = get_computing_device()
    help_str = (
        f'transcribe_anything is using a "{device}" device.'
        " Any unrecognized args are assumed to be for whisper"
        " ai and will be passed as is to whisper ai."
    )
    parser = argparse.ArgumentParser(description=help_str)
    parser.add_argument(
        "url_or_file",
        help="Provide file path or url (includes youtube/facebook/twitter/etc)",
        nargs="?",
    )
    parser.add_argument(
        "--query-gpu-json-path",
        help=(
            "Query the GPU and store it in the given path,"
            " warning takes a long time on first load!"
        ),
        type=Path,
    )
    parser.add_argument(
        "--output_dir",
        help="Provide output directory name,d efaults to the filename of the file.",
        default=None,
    )
    parser.add_argument(
        "--model",
        help="name of the Whisper model to us",
        default="small",
        choices=WHISPER_MODEL_OPTIONS,
    )
    parser.add_argument(
        "--task",
        help="whether to perform transcription or translation",
        default="transcribe",
        choices=whisper_options["task"],
    )
    parser.add_argument(
        "--language",
        help="language to the target audio is in, default None will auto-detect",
        default=None,
        choices=[None] + whisper_options["language"],
    )
    parser.add_argument(
        "--device",
        help="device to use for processing, None will auto select CUDA if available or else CPU",
        default=None,
        choices=[None, "cpu", "cuda", "insane"],
    )
    parser.add_argument(
        "--hf_token",
        help="huggingface token to use for downloading models",
        default=None,
    )
    parser.add_argument(
        "--save_hf_token",
        help="save huggingface token to a file for future use",
        action="store_true",
    )
    parser.add_argument(
        "--diarization_model",
        help=(
            "Name of the pretrained model/ checkpoint to perform diarization."
            + " (default: pyannote/speaker-diarization). Only works for --device insane."
        ),
        default="pyannote/speaker-diarization-3.1",
    )
    parser.add_argument(
        "--timestamp",
        help=(
            "Whisper supports both chunked as well as word level timestamps. (default: chunk)."
            + " Only works for --device insane."
        ),
        choices=["chunk", "word"],
        default=None,
    )
    parser.add_argument(
        "--embed",
        help="whether to embed the translation file into the output file",
        action="store_true",
    )
    # add extra options that are passed into the transcribe function
    args, unknown = parser.parse_known_args()
    if args.url_or_file is None and args.query_gpu_json_path is None:
        print("No file or url provided")
        parser.print_help()
        sys.exit(1)
    args.unknown = unknown
    return args


def main() -> int:
    """Main entry point for the command line tool."""
    args = parse_arguments()
    unknown = args.unknown
    if args.query_gpu_json_path is not None:
        from transcribe_anything.insanely_fast_whisper import get_cuda_info

        json_str = get_cuda_info().to_json_str()
        path: Path = args.query_gpu_json_path
        path.write_text(json_str, encoding="utf-8")
        return 0
    if args.model == "large-legacy":
        args.model = "large"
    elif args.model == "large":
        print(
            "Defaulting to large-v3 model for --model large,"
            + " use --model large-legacy for the old model"
        )
        args.model = "large-v3"
    elif args.model is None and args.device == "insane":
        print("Defaulting to large-v3 model for --device insane")
        args.model = "large-v3"

    hf_token_path = Path(user_cache_dir(), "hf_token.txt")
    if args.hf_token is None:
        args.hf_token = os.environ.get("HF_TOKEN", None)
        if args.hf_token is None and hf_token_path.exists():
            # read from file
            args.hf_token = hf_token_path.read_text(encoding="utf-8").strip() or None
        if args.hf_token is None:
            args.diarization_model = None
    if args.save_hf_token:
        hf_token_path.write_text(args.hf_token or "", encoding="utf-8")
        print("Saved huggingface token to", hf_token_path)

    # For now, just stuff --diarization_model and --timestamp into unknown
    if args.diarization_model:
        if args.device != "insane":
            print(
                "--diarization_model only works with --device insane. Ignoring --diarization_model"
            )
        else:
            unknown.append(f"--diarization_model {args.diarization_model}")

    if args.timestamp:
        if args.device != "insane":
            print("--timestamp only works with --device insane. Ignoring --timestamp")
        else:
            unknown.append(f"--timestamp {args.timestamp}")

    if unknown:
        print(f"Args passed to whisper backend: {unknown}")
    print(f"Running transcribe_audio on {args.url_or_file}")
    try:
        from transcribe_anything.api import transcribe

        transcribe(
            url_or_file=args.url_or_file,
            output_dir=args.output_dir,
            model=args.model if args.model != "None" else None,
            task=args.task,
            language=args.language if args.language != "None" else None,
            device=args.device,
            embed=args.embed,
            hugging_face_token=args.hf_token,
            other_args=unknown,
        )
    except KeyboardInterrupt:
        print("KeyboardInterrupt")
        return 1
    except Exception as e:  # pylint: disable=broad-except
        stack = traceback.format_exc()
        sys.stderr.write(f"Error: {e}\n{stack}\nwhile processing {args.url_or_file}\n")
        return 1
    return 0


if __name__ == "__main__":
    # push sys argv prior to call
    sys.argv.append("test.wav")
    sys.argv.append("--model")
    sys.argv.append("large")
    sys.argv.append('--initial_prompt "What is your name?"')
    sys.exit(main())
